import { Component, Input, OnInit, ViewChild } from '@angular/core';
import { MapGeocoder } from '@angular/google-maps';
import { Geolocation } from '@capacitor/geolocation';
import Chart from 'chart.js/auto';

@Component({
  selector: 'app-ree',
  templateUrl: './ree.component.html',
  styleUrls: ['./ree.component.scss'],
})
export class ReeComponent implements OnInit {
  position: any;
  @ViewChild('map') map: any;
  @ViewChild('topCountry') topCountry: any;
  @ViewChild('topReserv') topReserv: any;
  @ViewChild('topCompany') topCompany: any;
  @ViewChild('topMine') topMine: any;
  @ViewChild('popover') popover: any;
  page = 'infos';
  @Input() selected: any;
  topCountries: any;
  chartLoaded = false;
  markers: any[] = [];
  countryProd = 0;
  country?: string;
  max = 0;
  statsSegment = 'prod';
  statsVal: any;
  label = 'pays producteurs';

  constructor(private geocoder: MapGeocoder) {}

  async ngOnInit() {
    this.getKeys(this.selected.topCountries).forEach(
      async (country, index) =>
        (this.markers[index] = (
          await this.geocoder.geocode({ address: country }).toPromise()
        )?.results[0].geometry.location.toJSON())
    );
    this.statsVal = this.selected.topCountries;
    this.getMax();

    const coords = (await Geolocation.getCurrentPosition()).coords,
      lat = coords.latitude,
      lng = coords.longitude;
    this.position = { lat, lng };
  }

  ngDoCheck(e: any) {
    this.loadCharts();
  }

  loadCharts() {
    if (this.page === 'stats') {
      if (this.topCountry && !this.chartLoaded) {
        this.chartLoaded = true;
        this.topCountries = new Chart(this.topCountry.nativeElement, {
          type: 'bar',
          data: {
            labels: Object.keys(this.selected.topCountries),
            datasets: [
              {
                label: '',
                data: Object.values(this.selected.topCountries).map(
                  (val: any) => parseInt(val)
                ),
                backgroundColor: [
                  'fuchsia',
                  'gray',
                  'green',
                  'lime',
                  'maroon',
                  'navy',
                  'olive',
                  'orange',
                  'purple',
                  'red',
                  'silver',
                  'teal',
                  'yellow',
                ],
              },
            ],
          },
        });
        this.topReserv = new Chart(this.topReserv.nativeElement, {
          type: 'pie',
          data: {
            labels: Object.keys(this.selected.topReserves),
            datasets: [
              {
                data: Object.values(this.selected.topReserves).map((val: any) =>
                  parseInt(val)
                ),
              },
            ],
          },
        });
        this.topCompany = new Chart(this.topCompany.nativeElement, {
          type: 'line',
          data: {
            labels: Object.keys(this.selected.topCompanies),
            datasets: [
              {
                label: '',
                data: Object.values(this.selected.topCompanies).map(
                  (val: any) => parseInt(val)
                ),
                backgroundColor: [
                  'fuchsia',
                  'gray',
                  'green',
                  'lime',
                  'maroon',
                  'navy',
                  'olive',
                  'orange',
                  'purple',
                  'red',
                  'silver',
                  'teal',
                  'yellow',
                ],
              },
            ],
          },
        });
        this.topMine = new Chart(this.topMine.nativeElement, {
          type: 'polarArea',
          data: {
            labels: Object.keys(this.selected.topMines),
            datasets: [
              {
                data: Object.values(this.selected.topMines).map((val: any) =>
                  parseInt(val)
                ),
              },
            ],
          },
        });
      }
    } else {
      this.chartLoaded = false;
    }
  }

  getValues(obj: any) {
    return Object.values(obj).map((val: any) => parseInt(val));
  }

  getKeys(obj: any) {
    return Object.keys(obj);
  }

  test(i: number) {
    this.countryProd = parseInt(
      <any>this.getValues(this.selected.topCountries)[i]
    );
    this.country = this.getKeys(this.selected.topCountries)[i];
    console.log(this.popover.present());
  }

  setStatsValue() {
    switch (this.statsSegment) {
      case 'reserv':
        this.statsVal = this.selected.topReserves;
        this.label = 'réserves mondiales';
        break;
      case 'company':
        this.statsVal = this.selected.topCompanies;
        this.label = 'entreprises';
        break;
      case 'mines':
        this.statsVal = this.selected.topMines;
        this.label = 'mines';
        break;

      default:
        this.statsVal = this.selected.topCountries;
        this.label = 'pays producteurs';
        break;
    }
    this.getMax();
  }

  getMax() {
    const values = this.getValues(this.statsVal);
    this.max = values[values.indexOf(Math.max(...values))];
  }
}
